@@ -18,33 +18,37 @@ import logging as py_logging
 import time
 
 from oslo_log import log as logging
-import retrying
+from oslo_utils import encodeutils
+from oslo_utils import strutils
 import six
 
 from os_brick.i18n import _
 
+_time_sleep = time.sleep
+
+
+def _sleep(duration):
+    """Helper class to make it easier to work around tenacity's sleep calls.
+
+    Apparently we are all idiots for wanting to test our code here [0], so this
+    is a hack to be able to get retries to not actually sleep.
+
+    [0] https://github.com/jd/tenacity/issues/25
+    """
+    _time_sleep(duration)
+
+
+time.sleep = _sleep
+
+
+import tenacity  # noqa
+
 
 LOG = logging.getLogger(__name__)
 
 
 def retry(exceptions, interval=1, retries=3, backoff_rate=2):
 
-    def _retry_on_exception(e):
-        return isinstance(e, exceptions)
-
-    def _backoff_sleep(previous_attempt_number, delay_since_first_attempt_ms):
-        exp = backoff_rate ** previous_attempt_number
-        wait_for = max(0, interval * exp)
-        LOG.debug("Sleeping for %s seconds", wait_for)
-        return wait_for * 1000.0
-
-    def _print_stop(previous_attempt_number, delay_since_first_attempt_ms):
-        delay_since_first_attempt = delay_since_first_attempt_ms / 1000.0
-        LOG.debug("Failed attempt %s", previous_attempt_number)
-        LOG.debug("Have been at this for %s seconds",
-                  delay_since_first_attempt)
-        return previous_attempt_number == retries
-
     if retries < 1:
         raise ValueError(_('Retries must be greater than or '
                          'equal to 1 (received: %s). ') % retries)
@@ -53,9 +57,14 @@ def retry(exceptions, interval=1, retries=3, backoff_rate=2):
 
         @six.wraps(f)
         def _wrapper(*args, **kwargs):
-            r = retrying.Retrying(retry_on_exception=_retry_on_exception,
-                                  wait_func=_backoff_sleep,
-                                  stop_func=_print_stop)
+            r = tenacity.Retrying(
+                before_sleep=tenacity.before_sleep_log(LOG, logging.DEBUG),
+                after=tenacity.after_log(LOG, logging.DEBUG),
+                stop=tenacity.stop_after_attempt(retries),
+                reraise=True,
+                retry=tenacity.retry_if_exception_type(exceptions),
+                wait=tenacity.wait_exponential(
+                    multiplier=interval, min=0, exp_base=backoff_rate))
             return r.call(f, *args, **kwargs)
 
         return _wrapper
@@ -136,7 +145,12 @@ def trace(f):
 
         all_args = inspect.getcallargs(f, *args, **kwargs)
         logger.debug('==> %(func)s: call %(all_args)r',
-                     {'func': func_name, 'all_args': all_args})
+                     {'func': func_name,
+                      # NOTE(mriedem): We have to stringify the dict first
+                      # and don't use mask_dict_password because it results in
+                      # an infinite recursion failure.
+                      'all_args': strutils.mask_password(
+                          six.text_type(all_args))})
 
         start_time = time.time() * 1000
         try:
@@ -150,9 +164,34 @@ def trace(f):
             raise
         total_time = int(round(time.time() * 1000)) - start_time
 
+        if isinstance(result, dict):
+            mask_result = strutils.mask_dict_password(result)
+        elif isinstance(result, six.string_types):
+            mask_result = strutils.mask_password(result)
+        else:
+            mask_result = result
+
         logger.debug('<== %(func)s: return (%(time)dms) %(result)r',
                      {'func': func_name,
                       'time': total_time,
-                      'result': result})
+                      'result': mask_result})
         return result
     return trace_logging_wrapper
+
+
+def convert_str(text):
+    """Convert to native string.
+
+    Convert bytes and Unicode strings to native strings:
+
+    * convert to bytes on Python 2:
+      encode Unicode using encodeutils.safe_encode()
+    * convert to Unicode on Python 3: decode bytes from UTF-8
+    """
+    if six.PY2:
+        return encodeutils.to_utf8(text)
+    else:
+        if isinstance(text, bytes):
+            return text.decode('utf-8')
+        else:
+            return text
